local friends = LibStub("LibFriends-1.0")
local guild = LibStub("LibGuild-1.0")

local addon = LibStub("AceAddon-3.0"):NewAddon("Gratz", "AceConsole-3.0", "LibSink-2.0")

local db
function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("GratzDB", {
		profile = {
			friends = {
				['**'] = {announce = false, message = false, private = true, threshold = 1,},
				ding = {announce = true, text = "Gratz!",},
				connect = {text = "Welcome back!",},
				disconnect = {text = "Good riddance!",},
				add = {text = "I added you to my friends."},
				remove = {text = "We're not friends any more.",},
			},
			guild = {
				['**'] = {announce = false, message = false, private = false, threshold = 1,},
				ding = {announce = true, text = "Gratz!",},
				connect = {text = "Welcome back!",},
				disconnect = {text = "Good riddance!",},
				add = {announce = true, text = "Welcome to the guild!"},
				remove = {announce = true, text = "Good riddance.",},
			},
			sink = {
				sink20OutputSink = "ChatFrame",
			},
		},
	})
	db = self.db.profile
	self:SetSinkStorage(db.sink)
end

function addon:OnEnable()
	friends.RegisterCallback(self, "LevelUp", "Friend_LevelUp")
	friends.RegisterCallback(self, "Connected", "Friend_Connect")
	friends.RegisterCallback(self, "Disconnected", "Friend_Disconnect")
	friends.RegisterCallback(self, "Added", "Friend_Added")
	friends.RegisterCallback(self, "Removed", "Friend_Removed")
	guild.RegisterCallback(self, "LevelUp", "Guild_LevelUp")
	guild.RegisterCallback(self, "Connected", "Guild_Connect")
	guild.RegisterCallback(self, "Disconnected", "Guild_Disconnect")
	guild.RegisterCallback(self, "Added", "Guild_Added")
	guild.RegisterCallback(self, "Removed", "Guild_Removed")
end

function addon:OnDisable()
	friends.UnregisterAll(self)
	guild.UnregisterAll(self)
end

local function makeHandler(lib, eventName, eType, eKey, announce, channel)
	addon[eventName] = function(self, event, name, ...)
		local s = db[eType][eKey]
		local level = lib:GetLevel(name)
		if level and (level < s.threshold) then return end
		if s.announce then
			self:Pour((announce):format(name, ...))
		end
		if s.message and channel then
			channel = s.private and "WHISPER" or channel
			local message = (s.text):gsub("$N", name):gsub("$L", level or '?')
			SendChatMessage(message, channel, nil, channel == "WHISPER" and name or nil)
		end
	end
end

makeHandler(friends, "Friend_LevelUp", "friends", "ding", "%s dinged %d!", "WHISPER")
makeHandler(friends, "Friend_Connect", "friends", "connect", "%s logged on", "WHISPER")
makeHandler(friends, "Friend_Disconnect", "friends", "disconnect", "%s logged off", "WHISPER")
makeHandler(friends, "Friend_Added", "friends", "add", "%s added to friend list", "WHISPER")
makeHandler(friends, "Friend_Removed", "friends", "remove", "%s removed from friend list", false)
makeHandler(guild, "Guild_LevelUp", "guild", "ding", "%s dinged %d!", "GUILD")
makeHandler(guild, "Guild_Connect", "guild", "connect", "%s logged on", "GUILD")
makeHandler(guild, "Guild_Disconnect", "guild", "disconnect", "%s logged off", "GUILD")
makeHandler(guild, "Guild_Added", "guild", "add", "%s joined guild", "GUILD")
makeHandler(guild, "Guild_Removed", "guild", "remove", "%s left guild", "GUILD")

